package jp.co.sds_corp.fxchart.graph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.Map;

import javax.imageio.ImageIO;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.nanchart.graph.DefaultStorage;
import jp.co.sds_corp.nanchart.graph.Graph2D;
import jp.co.sds_corp.nanchart.graph.LineGraph;
import jp.co.sds_corp.nanchart.graph.RosokuGraph;
import jp.co.sds_corp.nanchart.graph.RosokuStorage;
import jp.co.sds_corp.nanchart.graph.XScale;
import jp.co.sds_corp.nanchart.graph.YScale;
import jp.co.sds_corp.util.common.CmnDate;

/**
* 時間足グラフ、5分足グラフを生成します。
*/
public class RateGraphHour extends RateGraph
{
	CmnDate	fr_data;
	CmnDate	dt_mon;
	CmnDate	dt_sat;
	int		time_diff;
	int		skip_hour = 3;
	int		skip_min = 15;
	boolean	jumpWeekend = false;;	// 週末を表示しないフラグ
	boolean	bJump = false;;	// 週末を表示しないフラグ
	boolean	bWeekend = false;;	// 週末フラグ

	protected void makeGraph( OutputStream out )
		throws Exception
	{
		CmnDate to = getTo();
		if ( to == null ) {
			to = new CmnDate();
			setTo( to );
		}
		dt_mon = getDayOfWeek( to, CmnDate.MONDAY );
		dt_sat = getDayOfWeek( to, CmnDate.SATURDAY );
		if ( dt_sat.after( dt_mon )) {
			bWeekend = true;
			time_diff = diffMinutes( to, dt_sat );
		} else {
			bWeekend = false;
			time_diff = diffMinutes( dt_mon, dt_sat );
		}

		bJump = jumpWeekend;
		if ( span.equals( "DAY" ) ) { //|| dt_mon.before( dt_sat )) {
			bJump = false;
		}

		CmnDate fr = getFrom();
		fr_data = getFrom();
		if ( fr == null ) {
			fr = (CmnDate )to.clone();
			fr_data = (CmnDate )to.clone();
			if ( span.equals( "5MIN" )) {
				fr.add( CmnDate.MINUTE, -100 * 5 );
			} else if ( span.equals( "HOUR" )) {
				fr.add( CmnDate.HOUR, -100 );
			}
			setFrom( fr );
		}

		fr_data = fr;
		if ( bJump && fr.before( dt_mon )) {
			fr_data = (CmnDate )dt_sat.clone();
			fr_data.add( CmnDate.MINUTE,
						0 - diffMinutes( to, fr ) + diffMinutes( to, dt_mon ));
		}
		if ( bJump && bWeekend ) {
			fr_data = (CmnDate )dt_sat.clone();
			fr_data.add( CmnDate.MINUTE,
						0 - diffMinutes( to, fr ) );
		}
/*
CmnLog.debug( this, "jump=" + bJump + " weekend=" + bWeekend );
CmnLog.debug( this, "fr     =" + fr.format( "yyyy-MM-dd HH:mm" ));
CmnLog.debug( this, "to     =" + to.format( "yyyy-MM-dd HH:mm" ));
CmnLog.debug( this, "fr_data=" + fr_data.format( "yyyy-MM-dd HH:mm" ));
CmnLog.debug( this, "dt_mon =" + dt_mon.format( "yyyy-MM-dd HH:mm" ));
CmnLog.debug( this, "dt_sat =" + dt_sat.format( "yyyy-MM-dd HH:mm" ));
*/

		Map<String,RateHistRosoku> mapHist = getSource().getData( fr_data, to, span );

		double	dMax = getSource().getMax();
		double	dMin = getSource().getMin();
/*
		if ( mapHist.size() == 0 ) {
			RateExchange re = new RateExchange( pair );
			re.loadCurrentRate();
			dMin = dMax = re.getFwdRate( "SP" ).l_rate.doubleValue();
		}
*/
		if ( quote != null ) {
			quote.start();
			double dTTM = -1.0;
			if ( quote.getStatus() == 0 && quote.isToday()) {
				dTTM = quote.getTTM();
				if ( dMin > dTTM ) {
					dMin = dTTM;
				}
				if ( dMax < dTTM ) {
					dMax = dTTM;
				}
			}
		}

		double  dDiv = Math.pow( 10, pair.getPoint() - 1 );
		dMin = ( Math.floor( dMin*dDiv )) / dDiv ;
		dMax = ( Math.floor( dMax*dDiv ) + 1.0 ) / dDiv ;

		Graph2D grf = new Graph2D( fr, to, new Double( dMin ), new Double( dMax ));

		RosokuGraph dat = createRosoku( mapHist, fr, to, dMin, dMax );
		grf.addSeries( dat );
		XScale  xs = new XScale( fr, to );
		setXScale( xs, span, fr, to );
		dat.addScale( xs );

		if ( bJump ) {
			RosokuGraph dat2 = createRosoku2( mapHist, fr, to, dMin, dMax );
			grf.addSeries( dat2 );
			XScale  xs2 = new XScale( fr, to );
			setXScale2( xs2, span, fr, to );
			dat2.addScale( xs2 );

			XScale  xs_sep = (XScale )grf.getXScale();
			setXScaleSeparator( xs_sep, fr );
		}

		if ( ma != null ) {
			LineGraph datma = createMa( mapHist, fr, to, dMin, dMax );
			grf.addSeries( datma );
		}

		YScale ys = new YScale( dMin, dMax );
		dDiv = Math.pow( 10, pair.getPoint() - 1 );
		setYScale( ys, dMin, dMax, dDiv, pair );
		dat.addScale( ys );

		YScale  ys2 = (YScale )grf.getYScale();
		setYScale2( ys2, dMin, dMax, pair, quote );

		Rectangle rect = new Rectangle( 35, 5, getWidth()-getMarginWidth()-10,
										getHeight()-getMarginHeight() );
		if ( getBgImage() != null ) {
			grf.setBgImage( getBgImage() );
		}
		grf.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
								RenderingHints.VALUE_ANTIALIAS_ON);
		BufferedImage img;
		img = grf.getImage( getWidth(), getHeight(), rect, BufferedImage.TYPE_INT_RGB );
		copyright( img, 0, getHeight() - 10 );

		ImageIO.write( img, "jpeg", out );
	}

	private int diffMinutes( CmnDate fr, CmnDate to ) {
		long lFr = fr.getTime().getTime();
		long lTo = to.getTime().getTime();
		return (int )(( lFr - lTo ) / 1000 / 60 );
	}

	private CmnDate getDayOfWeek( CmnDate to, int dow ) {
		CmnDate ret = (CmnDate )to.clone();
		ret.set( CmnDate.SECOND, 0 );
		ret.set( CmnDate.MINUTE, 0 );
		ret.set( CmnDate.HOUR_OF_DAY, 6 );

		for ( int i = 0; i < 7; i++ ) {
			if( ret.get( CmnDate.DAY_OF_WEEK ) == dow ) {
				break;
			}
			ret.add( CmnDate.DATE, -1 );
		}
		return ret;
	}

	protected RosokuGraph createRosoku( Map<String,RateHistRosoku> mapHist,
		CmnDate fr, CmnDate to, double dMin, double dMax )
	{
		RosokuGraph dat = new RosokuGraph( fr, to, dMin, dMax );
		RosokuStorage dt = (RosokuStorage)dat.getStorage();
		dat.setLineColor( Color.black );
		dat.setStroke( new BasicStroke( 1 ) );
		dat.setBarWidth( 2 );
		if ( bJump && bWeekend ) {
			return dat;
		}

		RateHistRosoku cur = null;
		Iterator<String> itr = mapHist.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			cur = (RateHistRosoku )mapHist.get( key );
			if ( bJump && cur.getStartDate().before( dt_mon )) {
				continue;
			}
			dt.add( cur.getStartDate(),
				cur.high.doubleValue(), cur.low.doubleValue(),
				cur.start.doubleValue(), cur.end.doubleValue() );
		}
		return dat;
	}

	protected RosokuGraph createRosoku2( Map<String,RateHistRosoku> mapHist,
		CmnDate fr, CmnDate to, double dMin, double dMax )
	{
		RosokuGraph dat = new RosokuGraph( fr, to, dMin, dMax );
		RosokuStorage dt = (RosokuStorage)dat.getStorage();
		dat.setLineColor( Color.black );
		dat.setStroke( new BasicStroke( 1 ) );
		dat.setBarWidth( 2 );

		RateHistRosoku cur = null;
		Iterator<String> itr = mapHist.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			cur = (RateHistRosoku )mapHist.get( key );
			if ( cur.getStartDate().after( dt_sat )) {
				break;
			}
			CmnDate tmp = (CmnDate )cur.getStartDate().clone();

			if ( span.equals( "5MIN" )) {
				tmp.add( CmnDate.MINUTE, time_diff - skip_min );
			} else if ( span.equals( "HOUR" )) {
				tmp.add( CmnDate.MINUTE, time_diff - ( skip_hour * 60 ));
			}
			dt.add( tmp,
				cur.high.doubleValue(), cur.low.doubleValue(),
				cur.start.doubleValue(), cur.end.doubleValue() );
		}
		return dat;
	}

	protected LineGraph createMa( Map<String,RateHistRosoku> mapHist,
		CmnDate fr, CmnDate to, double dMin, double dMax )
	{
		LineGraph dat =
			new LineGraph( fr, to, new Double(dMin), new Double(dMax) );
		DefaultStorage dt2 = (DefaultStorage)dat.getStorage();
		dat.setLineColor( Color.red );
		dat.setStroke( new BasicStroke( 1 ) );

		RateHistRosoku cur = null;
		Iterator<String> itr = mapHist.keySet().iterator();
		while ( itr.hasNext() ) {
			cur = (RateHistRosoku )mapHist.get( itr.next() );
			if ( cur.getEndDate().before( fr )) {
				continue;
			}
			double	dSum = 0.0;
			int nNum = 0;
			CmnDate shDt = getMaDate( cur.getEndDate() );
			Iterator<String> itj = mapHist.keySet().iterator();
			while ( itj.hasNext() ) {
				RateHistRosoku sh = (RateHistRosoku )mapHist.get( itj.next() );
				if ( sh.getEndDate().after( cur.getEndDate() )) {
					break;
				}
				if ( sh.getEndDate().before( shDt )) {
					continue;
				}
				dSum += sh.end.doubleValue();
				nNum++;
			}
			dt2.add( new CmnDate( cur.getEndDate().getTime()),
				dSum / nNum );
		}
		return dat;
	}

	protected CmnDate getMaDate( CmnDate dt ) {
		CmnDate ret = (CmnDate )dt.clone();
		if ( span.equals( "5MIN" )) {
			ret.add( CmnDate.MINUTE, 0 - Integer.valueOf( ma ).intValue()*5 );
		} else if ( span.equals( "HOUR" )) {
			ret.add( CmnDate.HOUR, 0 - Integer.valueOf( ma ).intValue() );
		}
		return ret;
	}

	protected void setXScale( XScale xs, String span, CmnDate fr, CmnDate to ) {
		Font font = new Font( "Dialog", Font.PLAIN, 8 );
		if ( bJump && bWeekend ) {
			return;
		}

		CmnDate tmp = (CmnDate )fr.clone();
		tmp.set( CmnDate.SECOND, 0 );
		tmp.set( CmnDate.MINUTE, 0 );
		for ( ;; ) {
			tmp.add( CmnDate.HOUR, 1 );
			if ( tmp.after( to ) ) {
				break;
			}
			if ( bJump && !tmp.after( dt_mon )) {
				continue;
			}
			if ( span.equals( "5MIN" )) {
				xs.add( tmp.clone(), tmp.format( "HH" ));
			} else if ( span.equals( "HOUR" )) {
				if (( tmp.get( CmnDate.HOUR ) % 12 ) == 0 ) {
					xs.add( tmp.clone(), tmp.format( "HH" ));
				}
			}
		}
		xs.setLineStroke( getLineStroke() );
		xs.setLineColor( getLineColor() );
		xs.setScaleLine( true );
		xs.setFont( font );
	}

	protected void setXScale2( XScale xs, String span, CmnDate fr, CmnDate to )
	{
		Font font = new Font( "Dialog", Font.PLAIN, 8 );

		CmnDate tmp = (CmnDate )fr_data.clone();
		tmp.set( CmnDate.SECOND, 0 );
		tmp.set( CmnDate.MINUTE, 0 );
		for ( ;; ) {
			tmp.add( CmnDate.HOUR, 1 );
			if ( tmp.after( to ) ) {
				break;
			}
			if ( !tmp.before( dt_sat )) {
				break;
			}
			if ( span.equals( "5MIN" )) {
				CmnDate tmp2 = (CmnDate )tmp.clone();
				tmp2.add( CmnDate.MINUTE, time_diff - skip_min );
				xs.add( tmp2, tmp.format( "HH" ));
			} else if ( span.equals( "HOUR" )) {
				if (( tmp.get( CmnDate.HOUR ) % 12 ) == 0 ) {
					CmnDate tmp2 = (CmnDate )tmp.clone();
					tmp2.add( CmnDate.MINUTE, time_diff - ( skip_hour * 60 ));
					xs.add( tmp2, tmp.format( "HH" ));
				}
			}
		}
		xs.setLineStroke( getLineStroke() );
		xs.setLineColor( getLineColor() );
		xs.setScaleLine( true );
		xs.setFont( font );
	}

	protected void setXScaleSeparator( XScale xs, CmnDate fr ) {
		CmnDate tmp = (CmnDate )dt_sat.clone();
		if ( span.equals( "5MIN" )) {
			tmp.add( CmnDate.MINUTE, time_diff );
		} else if ( span.equals( "HOUR" )) {
			tmp.add( CmnDate.MINUTE, time_diff );
		}
		xs.add( tmp, "" );
		tmp = (CmnDate )dt_sat.clone();
		if ( span.equals( "5MIN" )) {
			tmp.add( CmnDate.MINUTE, time_diff - skip_min );
		} else if ( span.equals( "HOUR" )) {
			tmp.add( CmnDate.MINUTE, time_diff - ( skip_hour * 60 ));
		}
		xs.add( tmp, "" );

		float[] dash = { 3 };
		xs.setLineStroke( new BasicStroke( 1,
				BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 1,
				dash, 0 ));
		xs.setLineColor( Color.black );
		xs.setScaleLine( true );
	}

	protected void setYScale2( YScale ys, double dMin, double dMax,
						CcyPair pair, Quote quote )
	{
		if ( quote == null || quote.getStatus() != 0 ) {
			return;
		}
		if ( quote.getTTM() > dMax || quote.getTTM() < dMin ) {
			return;
		}
		String pattern = "##0.0000000000".substring( 0, pair.getPoint()+4 );
		DecimalFormat df = new DecimalFormat( pattern );

		Font font = new Font( "Dialog", Font.PLAIN, 8 );
		ys.setFont( font );

		String sc;
		sc = df.format( quote.getTTM() );
		ys.add( quote.getTTM(), sc );

		ys.setLineStroke( new BasicStroke( 2 ));
		Color col = Color.red;
		if ( !quote.isToday()) {
			col = Color.gray;
		}
		ys.setLineColor( col );
		ys.setScaleLine( true );
	}

	/**
	* 週末を表示しないフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setJumpWeekend( boolean var ) {
		this.jumpWeekend = var;
	}
	/**
	* 週末を表示しないフラグを取得します。
	* @return	週末を表示しないフラグ
	*/
	public boolean getJumpWeekend() {
		return jumpWeekend;
	}

}
