package	jp.co.sds_corp.fxchart.service;

import java.io.File;
import java.io.FileOutputStream;

import javax.imageio.ImageIO;

import jp.co.sds_corp.fxchart.graph.DayFileSource;
import jp.co.sds_corp.fxchart.graph.HourFileSource;
import jp.co.sds_corp.fxchart.graph.IRateSource;
import jp.co.sds_corp.fxchart.graph.MonthFileSource;
import jp.co.sds_corp.fxchart.graph.Quote;
import jp.co.sds_corp.fxchart.graph.RateFileSource;
import jp.co.sds_corp.fxchart.graph.RateGraph;
import jp.co.sds_corp.fxchart.graph.RateGraphDay;
import jp.co.sds_corp.fxchart.graph.RateGraphHour;
import jp.co.sds_corp.fxchart.graph.RateGraphMonth;
import jp.co.sds_corp.fxchart.graph.RateGraphTick;
import jp.co.sds_corp.fxchart.graph.RateGraphWeek;
import jp.co.sds_corp.fxchart.graph.WeekFileSource;
import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.MasterCache;
import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;

/**
* チャート作成サービスです。
*/
public class GraphService extends AbstractService
{
	private static final long serialVersionUID = 1L;
	String		ccy;
	String		file;
	String		masterdir;
	String		ratedir;
	String		span;
	String		ma;
	String		bgimg;
	String		ttmfile;
	int			nWidth;
	int			nHeight;
	RateGraph	grf;
	IRateSource	src;
	MasterCache master = MasterCache.getInstance();

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		file = config.get( "file" );
		ratedir = config.get( "ratedir" );
		ccy = config.get( "ccy" );
		span = config.get( "type" );
		ma = config.get( "ma", null );
		String ccy1 = ccy.substring( 0, 3 );
		String ccy2 = ccy.substring( 3, 6 );
		CcyPair pair = master.getCcyPair( ccy1, ccy2 );

		if ( span.equals( "TICK" )) {
			src = new RateFileSource( pair, "SP", span );
			((RateFileSource )src).setDir( ratedir );
			grf = new RateGraphTick();
		} else if ( span.equals( "5MIN" )) {
			src = new RateFileSource( pair, "SP", span );
			((RateFileSource )src).setDir( ratedir );
			grf = new RateGraphHour();
			((RateGraphHour )grf).setJumpWeekend( true );
		} else if ( span.equals( "HOUR" )) {
			src = new HourFileSource( pair, "SP", span );
			((RateFileSource )src).setDir( ratedir );
			grf = new RateGraphHour();
			((RateGraphHour )grf).setJumpWeekend( true );
		} else if ( span.equals( "DAY" )) {
			src = new DayFileSource( pair, "SP", span );
			((DayFileSource )src).setDir( ratedir );
			grf = new RateGraphDay();
		} else if ( span.equals( "WEEK" )) {
			src = new WeekFileSource( pair, "SP", span );
			((WeekFileSource )src).setDir( ratedir );
			grf = new RateGraphWeek();
		} else if ( span.equals( "MONTH" )) {
			src = new MonthFileSource( pair, "SP", span );
			((MonthFileSource )src).setDir( ratedir );
			grf = new RateGraphMonth();
		} else {
			throw( new Exception( "illegal span:" + span ));
		}
		grf.setSource( src );
		grf.setPair( pair );
		grf.setSpan( span );
		grf.setMa( ma );

		try {
			ttmfile = config.get( "ttmfile" );
			Quote quote = new Quote();
			quote.setCcy( ccy1 );
			quote.setFile( ttmfile );
			grf.setQuote( quote );
		} catch ( Exception e ) {
		}
		try {
			bgimg = config.get( "bgimage" );
			//grf.setBgImage( new ImageIcon( bgimg ).getImage() );
			grf.setBgImage( ImageIO.read( new File( bgimg )) );
		} catch ( Exception e ) {
		}
		nWidth = Integer.valueOf( config.get( "width" )).intValue();
		nHeight = Integer.valueOf( config.get( "height" )).intValue();
		grf.setWidth( nWidth );
		grf.setHeight( nHeight );

	}

	/**
	* サービスを開始します。
	*/
	public void start() {
		setStatus( Status.RUN );
		try {
			CmnDate to = new CmnDate();
			grf.setTo( to );
			CmnDate fr = null;
			if ( span.equals( "5MIN" )) {
				fr = (CmnDate )to.clone();
				fr.add( CmnDate.HOUR, -8 );
			}
			grf.setFrom( fr );
			FileOutputStream fos = new FileOutputStream( file );
			grf.start( fos );
			fos.close();
			setStatus( Status.WAIT );
		} catch ( Throwable e ) {
			CmnLog.error( this, "make graph error ", e );
			setStatus( Status.ERROR );
		}
	}

	/**
	* サービスを終了します。
	*/
	public void stop() {
	}

	/**
	* このサービスの文字列表現を返します。
	*/
	public String toString() {
		String ret = super.toString();
		ret += " ccy=[" + ccy + "] span=[" + span + "] ";
		return ret;
	}
}
