package	jp.co.sds_corp.fxchart.service;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.PrintWriter;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.MasterCache;
import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.common.CmnString;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;

/**
* ローソク足グラフ用のデータを作成します。
*/
public class MakeCandle extends AbstractService
{
	private static final long serialVersionUID = 1L;
	CmnDate fromDt = null;
	String	ratedir;
	String	dayFile;
	String	dayTmp;
	String	hourFile;
	String	hourTmp;
	MasterCache master = MasterCache.getInstance();
	String target;

	private class History implements Cloneable {
		CcyPair		pair;
		String		term;
		BigDecimal  start;
		BigDecimal  end;
		BigDecimal  high;
		BigDecimal  low;
		CmnDate		startDate;
		CmnDate		endDate;
		protected Object clone() throws CloneNotSupportedException {
			return super.clone();
		}
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		target = config.get( "target", null );
		ratedir = config.get( "ratedir" );
		dayFile = ratedir + "/rate.day";
		// dayTmp = ratedir + "/rate.day.tmp";
		hourFile = ratedir + "/rate.hour";
		hourTmp = ratedir + "/rate.hour.tmp";
	}

	/**
	* サービスを開始します。
	*/
	public void start() {
		try {
			if ( target == null || target.equals( "today" )) {
				fromDt = new CmnDate();
				fromDt.add( CmnDate.DATE, -1 );
				fromDt.clearTime();
			} else if ( target.equals( "yesterday" )) {
				fromDt = new CmnDate();
				fromDt.add( CmnDate.DATE, -2 );
				fromDt.clearTime();
			} else {
				fromDt = new CmnDate( target, "yyyyMMdd" );
				fromDt.add( CmnDate.DATE, -1 );
				fromDt.clearTime();
			}

CmnLog.debug( this, "make temp from " + fromDt.getString() );
			CmnDate today = new CmnDate();
			CmnDate dt = (CmnDate )fromDt.clone();
			TreeMap<CmnDate,File> mapIn = new TreeMap<CmnDate,File>();
			for ( ;; ) {
				File file = new File( ratedir
										+ "/" + dt.format( "yyyyMM" )
										+ "/rate." + dt.format( "yyyyMMdd" ));
				CmnDate fdt = new CmnDate( file.getName().substring( 5 ));
				if ( fromDt.compareTo( fdt ) > 0 ) {
					continue;
				}
				if ( fdt.compareTo( today ) > 0 ) {
					break;
				}
				mapIn.put( fdt, file );
				dt.add( CmnDate.DATE, 1 );
			}

CmnLog.debug( this, "make day temp..." + fromDt.getString() );
			dayTmp = makeTmp( dayFile, "yyyyMMdd" );
CmnLog.debug( this, "make hour temp..." + fromDt.getString() );
			hourTmp = makeTmp( hourFile, "yyyyMMddHH" );

			Iterator<CmnDate> itr = mapIn.keySet().iterator();
			while ( itr.hasNext()) {
				CmnDate fdt = (CmnDate )itr.next();
				File f = (File )mapIn.get( fdt );
				readRate( f );
				writeDay();
				writeHour();
			}

			File tmp;
			File out;

			tmp = new File( dayTmp );
			out = new File( dayFile );
			tmp.renameTo( out );

			tmp = new File( hourTmp );
			out = new File( hourFile );
			tmp.renameTo( out );
			setStatus( Status.WAIT );
		} catch ( Exception e ) {
			CmnLog.error( this, "make day error", e );
			setStatus( Status.ERROR );
		}
	}

	/**
	* サービスを終了します。
	*/
	public void stop() {
	}

	private String makeTmp( String file, String dtFmt )
		throws Exception
	{
		File tmp = File.createTempFile( "rate", ".tmp", new File(ratedir));
		PrintWriter pw;
		pw = new PrintWriter( new FileOutputStream( tmp, false ));
		FileReader fr = new FileReader( file );
		BufferedReader in = new BufferedReader( fr );

		CmnDate dt = new CmnDate();
		for ( ;; ) {
			String line = in.readLine();
			if ( line == null ) {
				break;
			}
			String[] tok = CmnString.getStringToken( line, "\t" );
			if ( tok.length != 8 ) {
				continue;
			}
			dt.setDate( tok[7], dtFmt );
			if ( fromDt != null ) {
				if ( fromDt.compareTo( dt ) <= 0 ) {
					continue;
				}
			}
			pw.println( line );
		}

		pw.flush();
		pw.close();
		return tmp.getPath();
	}

	HashMap<String,History> mapDay = new HashMap<String,History>();
	TreeMap<String,History> mapHour = new TreeMap<String,History>();

	private void readRate( File inFile ) throws Exception {
		String term = "SP";
		mapDay.clear();
		mapHour.clear();

		CmnLog.debug( this, inFile.getName() );
		FileReader fr = null;
		BufferedReader in = null;
		try {
			fr = new FileReader( inFile.getPath() );
			in = new BufferedReader( fr );
		} catch ( Exception e ) {
			return;
		}

		for ( ;; ) {
			String line = in.readLine();
			if ( line == null ) {
				break;
			}
			String[] tok = CmnString.getStringToken( line, "\t" );
			if ( tok.length != 7 ) {
				CmnLog.error( this, "tok != 7:" + line );
				continue;
			}
			if( !tok[2].equals( term )) {
//				CmnLog.error( this, "term not:" + term +":"+ line );
				continue;
			}
			CcyPair pair = master.getCcyPair( tok[0], tok[1] );
			if ( pair == null ) {
				CmnLog.error( this, "pair not found:" + line );
				continue;
			}
			if( tok[3].length() == 0 ) {
				CmnLog.error( this, "tok[3]='':" + line );
				continue;
			}
			if( tok[4].length() == 0 ) {
				CmnLog.error( this, "tok[4]='':" + line );
				continue;
			}
			CmnDate dt = new CmnDate( tok[6], "yyyy-MM-dd HH:mm:ss" );
			BigDecimal l_rate = new BigDecimal( tok[3] )
						.setScale( pair.getPoint(), BigDecimal.ROUND_DOWN );
			BigDecimal r_rate = new BigDecimal( tok[4] )
						.setScale( pair.getPoint(), BigDecimal.ROUND_DOWN );
			BigDecimal mid = l_rate.add( r_rate )
						.divide( new BigDecimal( "2" ),
							pair.getPoint(), BigDecimal.ROUND_DOWN );

			History hist = new History();
			hist.pair = pair;
			hist.term = tok[2];
			hist.start = mid;
			hist.end = mid;
			hist.high = mid;
			hist.low = mid;
			hist.startDate = dt;
			hist.endDate = dt;

			String key1 = tok[0] + tok[1];
			updateHist( mapDay, key1, hist );

			String key2 = dt.format( "yyyyMMddHH" ) + tok[0] + tok[1];
			updateHist( mapHour, key2, hist );
		}
		in.close();
	}

	private void updateHist( Map<String,History> map, String key, History var ) throws Exception {
		History hist = (History )map.get( key );
		if ( hist == null ) {
			map.put( key, (History )var.clone() );
		} else {
			if ( hist.high.compareTo( var.high ) < 0 ) {
				hist.high = var.high;
			}
			if ( hist.low.compareTo( var.low ) > 0 ) {
				hist.low = var.low;
			}
			if ( hist.startDate.after( var.startDate )) {
				hist.startDate = var.startDate;
				hist.start = var.start;
			}
			if ( hist.endDate.before( var.endDate )) {
				hist.endDate = var.endDate;
				hist.end = var.end;
			}
		}
	}

	private void writeDay() throws Exception {
		PrintWriter pw = new PrintWriter( new FileOutputStream( dayTmp, true ));
		Iterator<String> itr = mapDay.keySet().iterator();
		while( itr.hasNext()) {
			String key = (String )itr.next();
			History hist = (History )mapDay.get( key );
			pw.println( hist.pair.getCcy1().getCode() + "\t"
						+ hist.pair.getCcy2().getCode() + "\t"
						+ hist.term + "\t"
						+ hist.high + "\t"
						+ hist.low + "\t"
						+ hist.start + "\t"
						+ hist.end + "\t"
						+ hist.startDate.format( "yyyyMMdd" ) );
		}
		pw.flush();
		pw.close();
	}

	private void writeHour() throws Exception {
		PrintWriter pw = new PrintWriter( new FileOutputStream( hourTmp, true));
		Iterator<String> itr = mapHour.keySet().iterator();
		while( itr.hasNext()) {
			String key = (String )itr.next();
			History hist = (History )mapHour.get( key );
			pw.println( hist.pair.getCcy1().getCode() + "\t"
						+ hist.pair.getCcy2().getCode() + "\t"
						+ hist.term + "\t"
						+ hist.high + "\t"
						+ hist.low + "\t"
						+ hist.start + "\t"
						+ hist.end + "\t"
						+ hist.startDate.format( "yyyyMMddHH" ) );
		}
		pw.flush();
		pw.close();
	}

}
